#!/bin/bash
# Autor: Arturo Martín Romero - amartinromero@gmail.com - programa de software libre

is_in_group() {
    
    USU="$1"
    GRUPO="$2"
    if id -Gn "$USU" | grep "$GRUPO" >/dev/null 2>&1 ; then
        return 0
    else
        return 1
    fi
}

## Comenzamos importando y definiendo las variables que usaremos posteriormente:
. /etc/default/vx-dga-variables/vx-dga-variables-general.conf

FICHFUNCS="/usr/bin/vx-funcs-bash"
[ -f "${FICHFUNCS}" ] && . "${FICHFUNCS}"


LOG="nfs-cliente.log"
vx-log_xsession -o "${LOG}"

## Flag en log LOG general
## Magenta titulo1, azul titulo2, rojo error, verde ok
vx-colorear_echo "magenta" "[$(vx-fecha)] NFS-SHARE: Inicio servicio de carpetas compartidas" | sudo vx-auditar-syslog


NFSRECURSOS="/usr/share/vitalinux/nfs-compartir/nfs-recursos"
if ! test -f ${NFSRECURSOS} ; then
    exit 1
fi
NFSEXCLUIDOS="/usr/share/vitalinux/nfs-compartir/nfs-excluidos"

USUARIO=$(vx-usuario-grafico)

# new migasfree-tag chequea todo...
ETIQUETAS=$(migasfree-tags -g)

echo "$(date) - << Etiquetas Migasfree: ${ETIQUETAS}"

if test -f /usr/bin/nfs-agregar-montaje.sh ; then
    echo "--> Comprobamos si hay algún nuevo recurso compartido que haya que agregar ..."
    /usr/bin/nfs-agregar-montaje.sh "${ETIQUETAS}"
fi

# Variable encargada de decidir si se muestran mensajes para evitar mensajes "pesados":
#  * Si en algún momento se pierde conexión con el servidor caché o no hay conexión se deshabilitan los mensajes
#  * Si hay un error en algún montaje también se deshabilitan los mensajes
MOSTRAR_MENSAJE=1

# Opciones de montaje por defecto:
MNTOPTIONSDFL="actimeo=1800,noatime,bg,nfsvers=3,tcp,rw,hard,intr,nodev,nosuid,exec"

# DOS OPCIONES:
# 1. Sencilla: creamos una variable temporal con los recursos a montar y no nos andamos haciéndolo cada vez en el bucle. Usamos lo que hay
# 2. Usamos el paquete de centro -> Problema: no todos lo tienen. Habría que ver si está o no.

# Obtenemos los recursos a montar, excluyendo los correspondientes y los dejamos en RECURSSOSMONTAR
declare -a RECURSOSMONTAR

# Cargamos el array
while IFS= read -r RECURSO
do
    CARPETAMONTAJE=$(echo "${RECURSO}" | cut -d":" -f2)
    TIPOUSUARIO=$(echo "${RECURSO}" | cut -d":" -f5)
    while IFS= read -r LINEAEXC
    do
        FLAGMONTAR=true
        CENTROEXCLUIDO=$(echo "${LINEAEXC}" | cut -d":" -f1)
        MONTAJESEXCLUIDOS=$(echo "${LINEAEXC}" | cut -d":" -f2)
        GRUPOEXCLUIDOS=$(echo "${LINEAEXC}" | cut -d":" -f3)
        if ( echo "${ETIQUETAS}" | grep "${CENTROEXCLUIDO}" &> /dev/null ) ; then
            echo "--> Este centro tiene excluidos: ${CENTROEXCLUIDO} -- ${ETIQUETAS}"
            if [ "${GRUPOEXCLUIDOS}" = "ALL" ] \
            || (is_in_group "${USUARIO}" "${GRUPOEXCLUIDOS}") ; then
                echo "--> El usuario tiene exclusiones de grupo: ${USUARIO} -- ${GRUPOEXCLUIDOS}"
                if [ "${MONTAJESEXCLUIDOS}" = "ALL" ] \
                || ( echo "${MONTAJESEXCLUIDOS}" | grep "${CARPETAMONTAJE}" &> /dev/null ); then
                    echo "--> Recurso excluido: ${CARPETAMONTAJE} -- ${MONTAJESEXCLUIDOS}"
                    FLAGMONTAR=false
                    break
                fi
            fi
        fi
    done < <(grep -v '^ *#' < "${NFSEXCLUIDOS}")
    # Exclusiones por tipo de usuario
    if [ "${TIPOUSUARIO}" = "ADM" ] && \
    ! is_in_group "${USUARIO}" "sudo" && \
    ! is_in_group "${USUARIO}" "profesor" && \
    ! is_in_group "${USUARIO}" "privado" ; then
        FLAGMONTAR=false
    fi
    if ${FLAGMONTAR}; then
        RECURSOSMONTAR+=("$RECURSO")
    else
        ## Comprobar si está montado...y si lo está desmontar
        grep "^${IPCACHE}:${RECURSOREMOTO}" /proc/mounts &> /dev/null
        MONTADO=$?
        if [ $MONTADO -eq 0 ]; then
            # Desmontar el recurso
            if umount -lf "${CARPETAMONTAJE}" ; then
                [ -z "$(ls -A "${CARPETAMONTAJE}" )" ] && rmdir --ignore-fail-on-non-empty "${CARPETAMONTAJE}"
                vx-colorear_echo "rojo" "Desmontado: $CARPETAMONTAJE...no debería estar montado"
            fi
        fi
    fi
done < <(grep -v '^ *#' < "${NFSRECURSOS}")

# En el caso de salir de forma inespearada desmontamos lo que haya montado
trap '/usr/bin/nfs-desmontajes.sh "${RECURSOSMONTAR[@]}"; exit' EXIT

while true ; do
    
    # Comprobamos si el servidor Caché esta presente y da servicio NFS cada xx segundos por si hay caída
    if ( ping -c 1 "${IPCACHE}" && echo >/dev/tcp/"${IPCACHE}"/2049 ) >/dev/null 2>&1 ; then
        for RECURSO in "${RECURSOSMONTAR[@]}"; do
            RECURSOREMOTO=$(echo "${RECURSO}" | cut -d":" -f1)
            CARPETAMONTAJE=$(echo "${RECURSO}" | cut -d":" -f2)
            MENSAJEOK=$(echo "${RECURSO}" | cut -d":" -f3)
            MENSAJEERROR=$(echo "${RECURSO}" | cut -d":" -f4)
            TIPOUSUARIO=$(echo "${RECURSO}" | cut -d":" -f5)
            
            MNTOPTIONS=$MNTOPTIONSDFL
            
            # ...comprobamos si estuviera montado y no debería
            grep "^${IPCACHE}:${RECURSOREMOTO}" /proc/mounts &> /dev/null
            MONTADO=$?
            showmount -e "${IPCACHE}" | grep "${RECURSOREMOTO}" &> /dev/null
            ACCESIBLE=$?
            
            if [ $MONTADO -eq 0 ] && [ $ACCESIBLE -ne 0 ]; then
                if umount -lf "${CARPETAMONTAJE}" ; then
                    [ -z "$(ls -A "${CARPETAMONTAJE}" )" ] && rmdir --ignore-fail-on-non-empty "${CARPETAMONTAJE}"
                    vx-colorear_echo "rojo" "$(date) - Desmontado: $CARPETAMONTAJE...Algo ha pasado con el recurso compartido" && \
                    [ "${MOSTRAR_MENSAJE}" = "1" ] && notify-send -t 1000 -i vx-dga-incorrecto "Algo ha pasado con un recurso compartido" && \
                    MOSTRAR_MENSAJE=0
                fi
                elif [ $MONTADO -ne 0 ] && [ $ACCESIBLE -eq 0 ]; then
                if /usr/bin/nfs-crear-directorios-montaje.sh "${CARPETAMONTAJE}" && \
                mount -t nfs "${IPCACHE}:${RECURSOREMOTO}" "${CARPETAMONTAJE}" -o "${MNTOPTIONS}"; then
                    vx-colorear_echo "azul" "Se monta el recurso ${RECURSOREMOTO}" && \
                    [ "${MOSTRAR_MENSAJE}" = "1" ] && notify-send -t 1000 -i vx-dga-correcto "${MENSAJEOK}"
                else
                    vx-colorear_echo "rojo" "$(date) - Error al montar el recurso ${RECURSOREMOTO}" && \
                    [ "${MOSTRAR_MENSAJE}" = "1" ] && notify-send -t 1000 -i vx-dga-incorrecto "${MENSAJEERROR}" && \
                    MOSTRAR_MENSAJE=0
                    
                fi
                
            fi
        done
    else
        # Se ha perdido el acceso al servidor caché...Desmontamos en el caso de estar montados
        # Sería conveniente un flag que salte ésta parte para mayor celeridad
        #MENSAJE_PERDIDA_CONEXION="Perdida la conexión con los recursos compartidos del servidor caché. Se intentará reconectar en breve"
        
        # Si tenemos problemas de acceso con el servidor caché, cancelamos el mostrar mensajes a partir de ahora...
        MOSTRAR_MENSAJE=0
        
        /usr/bin/nfs-desmontajes.sh "${RECURSOSMONTAR[@]}"
        # Como se quiere limpieza en el escritorio, se omiten notifiaciones
        #echo "$(date) - Se desmontan los recursos" && \
        #[ "${MOSTRAR_MENSAJE}" = "1" ] && \
        #notify-send -i vx-dga-incorrecto "${MENSAJE_PERDIDA_CONEXION}" && \
        #MOSTRAR_MENSAJE=0 && TEMPORIZADOR1="$(date +%s)" && TEMPORIZADOR2="$((${TEMPORIZADOR1} + (15 * 60)))"
    fi
    # Por si se produjera una desconexión inesperada, cada 15 segundos lo revisamos
    sleep 15
    
    # Comprobamos si se ha evitado mostrar mensajes durante los últimos 15 minutos...se deshabilita por limpieza
    
    # if [ "${MOSTRAR_MENSAJE}" = "0" ] ; then
    # 	TEMPORIZADOR1="$(date +%s)"
    # 	if [[ "${TEMPORIZADOR1}" > "${TEMPORIZADOR2}" ]] ; then
    # 		MOSTRAR_MENSAJE=1
    # 	fi
    # fi
    
done
